#!/usr/bin/env python
"""
Python module to try to handle distances between two close locations.

     Copyright (C) 2005  Kurt Schwehr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: projections.py,v 1.4 2005/08/22 20:20:44 schwehr Exp $

This module is just a hack/prototype for now.  The real solution is to
use something like proj.4 and/or talk to someone who actually knows
projections and datums.
"""

import math

def degMinSec2decDeg(deg,min,sec):
    """
    Convert degrees, minutes, seconds to decimal degrees


    >>> degMinSec2decDeg(-121,29,05.54)


    """
    if (0>deg): return (deg - min/60. - sec/3600.)
    return (deg + min/60. + sec/3600.)

def decimalDeg2degMinSec(angle):
    """
    Convert decimal degrees to Degrees, Minutes, Seconds..

    FIX: need a function that makes a pretty print version
    Optionally a LaTeX long lat output would be cool too!

    -120$^{\circ}$ 28' 27.06'', 34.0$^{\circ}$ 15' 52.01''
    or
    120$^{\circ}$ 28' 27.06'' W, 34.0$^{\circ}$ 15' 52.01'' N

    and be able to specify which one first
    """
    if (0>angle):
        deg = math.ceil(angle)
    else: deg = math.floor(angle)
    newAngle = math.fabs(angle - deg)
    min = math.floor(newAngle * 60.)
    sec = (newAngle - min/60.) * 3600.
    return deg,min,sec


def lonlat2utm(lon, lat, datum='wgs84'):
    """
    returns easting, northing, zone
    """
    assert (False and "Code this up")
    return False

def utm2lonlat(easting, northing, zone=None):
    """ Figure the zone ourselves if None"""
    assert (False and "Code this up")
    return False

def greatCircleDistMeters(lonlat1, lonlat2):
    """
    http://en.wikipedia.org/wiki/Great_circle_distance
    """
    return

def deg2rad(degrees):
    return (degrees / 180.) * math.pi
def rad2deg(radians):
    return (radians / math.pi) * 180.


def greatCircleDistKm(lonlat1, lonlat2):
    """
    Distance for far apart objects (0.5 % error)
    
    http://en.wikipedia.org/wiki/Great_circle_distance
    """
    a1 = deg2rad(float(lonlat1[1])); b1 = deg2rad(float(lonlat1[0]))
    a2 = deg2rad(float(lonlat2[1])); b2 = deg2rad(float(lonlat2[0]))
    r = 6367 # Earth radius in km
    from math import sin,acos,cos
    val1 = acos(sin(a1)*sin(a2)+cos(a1)*cos(a2)*cos(b1-b2))
    #print 'val1 = ',val1
    val = val1 * r
    return val


def getDistance(lonlat1, lonlat2):
    """
    return the distance in meters between two lon,lat tuples
    """
    return

def utmzone(lon,lat=0.):
    """
    Return the utm zone number
    
    http://www.dmap.co.uk/utmworld.htm
    """
    return

def utmzone2(lon,lat):
    """ Also returns the letter for the zone

    http://www.dmap.co.uk/utmworld.htm
    """

    
######################################################################
import unittest
class TestDistance(unittest.TestCase):
    def testWikipedia(self):
        """http://en.wikipedia.org/wiki/Great_circle_distance
        BNA 36.12  -86.67
        LAX 33.94 -118.40
        """
        BNA_DEG = (-86.67,36.12) # Lon,Lat degrees of Nashville
        LAX_DEG = (-118.40,33.94) # Los Angeles airport
        dist = greatCircleDistKm(BNA_DEG, LAX_DEG)
        print 'dist = ', dist
        self.failUnlessAlmostEqual(2884.6,dist,1)
        return
    def testUtmZone(self):
        # LAX & San Diego zone = 11S
        # SFO = 10S
        # UNH = 19T
        return
    def testDegDec2DegMinSec(self):
        self.failUnlessAlmostEqual(-121.,degMinSec2decDeg(-121,0,0),2)
        self.failUnlessAlmostEqual(-121.5,degMinSec2decDeg(-121,30,0),2)
        self.failUnlessAlmostEqual(-121.5083333,degMinSec2decDeg(-121,30,30),5)
        self.failUnlessAlmostEqual(-121.484872222222,degMinSec2decDeg(-121,29,05.54),6)
        deg, min, sec = decimalDeg2degMinSec(-121.484872222222)
        print "deg,min,sec: ",deg,min,sec
        self.failUnlessEqual(int(-121),   int(deg))
        self.failUnlessAlmostEqual(  29,   min,1)
        self.failUnlessAlmostEqual(  05.54,sec,2)
        return



if __name__ == '__main__':
    print "Testing distance"

    # BPSIO CORES in the Santa Barbara Basin
    corelocs = [(0,0),
                (-120.108,34.36116666),
                (-120.1075,34.36999999),
                (-120.05883333,34.36566666),
                (-120.05716666,34.37899999),
                (-120.02356666,34.33704999),
                (-119.97723333,34.32998333),
                (-120.03666666666666,34.287500000000001), # "ODP 893A"
    ]
    for p in corelocs[1:]:#[:-1]:
        print "  ",p,"  ", decimalDeg2degMinSec(p[0]), "  ", decimalDeg2degMinSec(p[1])

    d = greatCircleDistKm(corelocs[1],corelocs[2]); print 'dist 12', d, d*1000
    d = greatCircleDistKm(corelocs[3],corelocs[4]); print 'dist 34', d, d*1000
    d = greatCircleDistKm(corelocs[5],corelocs[6]); print 'dist 56', d, d*1000

    unittest.main()
